/*
   game/game.h
   This file is part of the Osirion project and is distributed under 
   the terms of the GNU General Public License version 2 
*/

#ifndef __INCLUDED_GAME_H__
#define __INCLUDED_GAME_H__

#include <vector>
#include <string>

#include "filesystem/inifile.h"
#include "game/ship.h"
#include "game/star.h"
#include "core/core.h"
#include "sys/sys.h"

/// the game-specific engine
/** The main game functions.
 */
namespace game
{

// entity type constants
const unsigned int ship_enttype = 256;
const unsigned int star_enttype = 257;
const unsigned int planet_enttype = 258;
const unsigned int navpoint_enttype = 259;
const unsigned int jumppoint_enttype = 260;

class Game : public core::Module {
public:
	Game();
	~Game();

	/// initialize the game
	void init();

	/// shutdown the game
	void shutdown();

	/// run one time frame
	void frame(float seconds);

	/// is called when a player connects
	void player_connect(core::Player *player);

	/// is called when a player disconnects
	void player_disconnect(core::Player *player);

	static inline Game *instance() { return game_instance; }

	core::Cvar *g_impulsespeed;
	core::Cvar *g_impulseacceleration;
	core::Cvar *g_strafespeed;
	core::Cvar *g_jumppointrange;

	core::Cvar *g_devel;

private:
	bool got_entity_key(filesystem::IniFile &inifile, core::Entity *entity);

	bool load_world();

	bool load_zone(core::Zone *zone);

	bool validate_zone(core::Zone *zone);

	bool load_ships();

	static Game *game_instance;
};

}

#endif // __INCLUDED_GAME_H__

