/*
   server/console.h
   This file is part of the Osirion project and is distributed under 
   the terms of the GNU General Public License version 2 
*/

#ifndef __INCLUDED_SERVER_CONSOLE_H__
#define __INCLUDED_SERVER_CONSOLE_H__

#include "sys/consoleinterface.h"

namespace server {

class Console : public sys::ConsoleInterface {
public:
	/// initialize the server console
	static void init();
	/// shutdown the server console
	static void shutdown();

#ifdef HAVE_CURSES
	/// flush buffered messages
	virtual void flush();
	/// resize the console
	virtual void resize();
	/// run one console frame
	void frame(float seconds);
protected:
	/// draw the ncurses console
	void draw();
	/// clear and draw background (ncurses)
	void draw_background();
	/// draw status
	void draw_status();
	/// draw the console text (ncurses)
	void draw_text();
	/// draw the console input (ncurses)
	void draw_input();
	/// dump console content to cout
	void dump();

private:
	/// set ncurses drawing color
	void set_color(const char *color_code);
	/// timestamp for screen refresh timeout
	float console_lastrefresh;
	
        // input history
        std::deque<std::string> history;
        std::deque<std::string>::reverse_iterator history_pos;

        size_t input_pos;
	
#endif
};

Console *console();

}

#endif // __INCLUDED_SERVER_CONSOLE_H__

