/*
   sys/sys.cc
   This file is part of the Osirion project and is distributed under
   the terms of the GNU General Public License version 2
*/


#ifdef _WIN32

#include <windows.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <direct.h>

#else

#include <unistd.h>
#include <signal.h>
#include <string.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>

#endif

#include <stdio.h>
#include <stdlib.h>
#include <string>

#include "sys/sys.h"

namespace sys {

bool isdirectory(std::string const &path)
{
#ifdef _WIN32
	struct ::_stat path_stat;
	memset(&path_stat, 0, sizeof(struct ::_stat));

	if (::_stat(path.c_str(), &path_stat) != 0) {
		return false;
	}

	if (path_stat.st_mode & _S_IFDIR) {
		return true;
	}
	return false;
#else
	struct stat path_stat;
	memset(&path_stat, 0, sizeof(path_stat));

	if (stat(path.c_str(), &path_stat) != 0) {
		return false;
	}

	if (path_stat.st_mode & S_IFDIR) {
		return true;
	}
	
	return false;
#endif
}

void mkdir(std::string const &path)
{
#ifdef _WIN32
	std::string p(path);
	for (size_t i = 0; i < p.size(); i++)
		if (p[i] == '/') p[i] = '\\';
	if (p.size() && (p[p.size()-1] == '\\'))
		p.erase(p.size() -1, 1);

	if (_mkdir(p.c_str()) != 0) {
		con_warn << "Could not create directory '" << p << "'" << std::endl;
	}
#else
	
	::mkdir(path.c_str(), 0777);
#endif
}

void signal(int signum, signalfunc handler)
{
#ifndef _WIN32
	struct sigaction sa;

	sa.sa_sigaction = 0;
	memset(&sa.sa_mask, 0 ,sizeof(sigset_t));
	sa.sa_flags = 0;
	sa.sa_handler = handler;

	::sigaction(signum, &sa, 0);
#endif
}

unsigned long time() 
{
#ifndef _WIN32
	struct ::tm localtime;
	time_t epochtime = ::time(0);
	::localtime_r(&epochtime, &localtime);
	return ((unsigned long) (localtime.tm_sec + localtime.tm_min*60 + localtime.tm_hour*3600));
#else
	return 0;
#endif
}

void sleep(float seconds)
{
#ifndef _WIN32
	::usleep((useconds_t) (seconds * 1000000.0f) );
#else
	Sleep((DWORD) (seconds*1000.0f));
#endif
}

void quit(int status) {
	::exit(status);
}

}

